#!/usr/bin/env python2
# coding: utf-8

import gi
gi.require_version("Gtk", "2.0")
gi.require_version('MatePanelApplet', '4.0')
gi.require_version('Bamf', '3')
 
from gi.repository import Gtk, GLib, GdkPixbuf, Gdk, GObject
from gi.repository import MatePanelApplet
from gi.repository import Gio

from plugins.kylinmenu import pluginclass

try:
    import sys
    import os
    import commands
    import gettext
    import traceback
    import time
    import gc
    import ctypes
    from ctypes import *
    import xdg.Config
    import keybinding
    import pointerMonitor
except Exception, e:
    print e
    sys.exit( 1 )
GObject.threads_init()

gdk = CDLL("libgdk-x11-2.0.so.0")
gtk = CDLL("libgtk-x11-2.0.so.0")
# i18n
gettext.install("kylin-menu", "/usr/share/locale")

NAME = _("Menu")
PATH = os.path.abspath( os.path.dirname( sys.argv[0] ) )

sys.path.append( os.path.join( PATH , "plugins") )

windowManager = os.getenv("DESKTOP_SESSION")
if not windowManager:
    windowManager = "MATE"
xdg.Config.setWindowManager( windowManager.upper() )

from execute import *

class MainWindow( object ):
    """This is the main class for the application"""

    def __init__( self, keybinder, settings ):
        self.keybinder = keybinder
        self.settings = settings
        self.path = PATH
        sys.path.append( os.path.join( self.path, "plugins") )

        self.de = "mate"

        # Load UI file and extract widgets   
        builder = Gtk.Builder()
        builder.add_from_file(os.path.join( self.path, "mate-menu.glade" ))
        self.window     = builder.get_object( "mainWindow" )
        self.window.set_name("startup-window")
        opacity = self.settings.get_double("opacity")
        self.window.set_opacity(opacity)
        self.eventbox   = builder.get_object( "eventbox1" )
        self.paneholder = builder.get_object( "paneholder" )
        self.border     = builder.get_object( "border" )
        
        builder.connect_signals(self)

        self.borderwidth = 1
        self.border.set_padding( self.borderwidth, self.borderwidth, self.borderwidth, self.borderwidth )
        
	defaultStyle = self.getDefaultStyle()

        self.window.connect( "key-press-event", self.onKeyPress )
        self.window.connect( "focus-in-event", self.onFocusIn )
        self.loseFocusId = self.window.connect( "focus-out-event", self.onFocusOut )
        self.loseFocusBlocked = False

        self.offset = 0

        self.tooltips = Gtk.Tooltips()
        self.tooltips.enable()

        self.PopulatePlugins()            
#        GLib.timeout_add( 200, self.PopulatePlugins )
        self.firstTime = True;

    def PopulatePlugins( self ):
        PluginPane = Gtk.EventBox()
        PluginPane.show()
        PaneLadder = Gtk.VBox( False, 0 )
        PluginPane.add( PaneLadder )
        ImageBox = Gtk.EventBox()
        ImageBox.show()

        self.plugins = {}
        self.pluginlist = {'kylinmenu'}

        self.showCategoryMenu = self.settings.get_boolean("show-category-menu")

        try:
            MyPlugin = pluginclass(self, self.showCategoryMenu)
        except Exception as e:
            print (e)
        MyPlugin.content_holder.show()

        VBox1 = Gtk.VBox( False, 0 )
        VBox1.show()
        #Add plugin to Plugin Box under heading button
        MyPlugin.content_holder.reparent( VBox1 )

        #Add plugin to main window
        PaneLadder.pack_start( VBox1 , True, True, 0)
        PaneLadder.show()

        if MyPlugin.window:
            MyPlugin.window.destroy()

        try:
            if hasattr( MyPlugin, 'do_plugin' ):
                MyPlugin.do_plugin()

            heightPath = os.path.join(GLib.get_home_dir(), ".windowHeight")
            if os.path.exists(heightPath):
                f = open(heightPath, "r")
                lines = f.readlines()
                length = lines[0]
                MyPlugin.windowHeight = int(length)
                f.close()
            MyPlugin.content_holder.set_size_request(345, MyPlugin.windowHeight )
        except Exception, e:
            print u"111111111111111111111111111"

        self.plugins['kylinmenu'] = MyPlugin

        self.paneholder.pack_start( ImageBox, False, False, 0 )
        self.paneholder.pack_start( PluginPane, False, False, 0 )
        self.tooltips.disable()

    # A little hacky but works
    def getDefaultStyle( self ):
        widget = Gtk.EventBox()
        widget.show()
        return Gtk.rc_get_style(widget)

    def RegenPlugins( self, *args, **kargs ):
        for item in self.paneholder:
            item.destroy()

        for plugin in list(self.plugins.values()):
            if hasattr( plugin, "destroy" ):
                plugin.destroy()

        try:
            del plugin
        except:
            pass

        try:
            del self.plugins
        except:
            pass

        gc.collect()

        self.PopulatePlugins()
    
    def setTooltip( self, widget, tip, tipPrivate = None ):
        self.tooltips.set_tip( widget, tip, tipPrivate )

    def onKeyPress( self, widget, event ):
        if event.keyval == Gdk.KEY_Escape:
            self.hide()
            return True
        return False

    def show( self ):
            
        for plugin in self.plugins.values():
            if hasattr( plugin, "onShowMenu" ):
                plugin.onShowMenu()

        self.window.present()

        
        self.window.window.focus( Gdk.CURRENT_TIME )

        for plugin in self.plugins.values():
            if hasattr( plugin, "changeTab" ):
                plugin.changeTab( 0 )


        Gdk.flush()

    def hide( self, forceHide = False ):
        for plugin in self.plugins.values():
            if hasattr( plugin, "onHideMenu" ):
                plugin.onHideMenu()
                
        self.window.hide()
        
    def onFocusIn( self, *args ):
        if self.loseFocusBlocked:
            self.window.handler_unblock( self.loseFocusId )
            self.loseFocusBlocked = False

        return False  
        
    def onFocusOut( self, *args):            
        if self.window.get_visible():
            self.hide()
        return False
        
    def stopHiding( self ):
        if not self.loseFocusBlocked:
            self.window.handler_block( self.loseFocusId )
            self.loseFocusBlocked = True

class MenuWin( object ):
    def __init__( self, applet, iid ):
        self.applet = applet
        self.mainwin = None

        self.createPanelButton()
        self.applet.set_flags( MatePanelApplet.AppletFlags.EXPAND_MINOR )
        self.applet.connect( "button-press-event", self.showMenu )
        GLib.timeout_add(100, self.InitMenu )

    def InitMenu( self ):
        if not os.path.exists( os.path.join( os.path.expanduser( "~" ), ".local", "share", "applications" ) ):
            os.makedirs( os.path.join( os.path.expanduser( "~" ), ".local", "share", "applications" ) )

        self.settings = Gio.Settings.new("org.ubuntu-mate.matemenu")
        self.settings.connect("changed::show-category-menu", self.changeMenuState)
        self.settings.connect("changed::permission-changed", self.changeMenuState)
        self.settings.connect("changed::opacity", self.changeOpacity)

        self.state = self.settings.get_boolean("show-category-menu")

        self.keybinder = keybinding.GlobalKeyBinding()
        self.hotkeyText =  "Super_L"
        self.mainwin = MainWindow( self.keybinder, self.settings )
        self.mainwin.window.connect( "map-event", self.onWindowMap )
        self.mainwin.window.connect( "unmap-event", self.onWindowUnmap )
        self.mainwin.window.connect( "realize", self.onRealize )
        self.mainwin.window.connect( "size-allocate", lambda *args: self.positionMenu() )

        if self.keybinder is not None:
            self.bind_hot_key()
        self.applet.set_can_focus(False)

        self.pointerMonitor = pointerMonitor.PointerMonitor()
        if self.pointerMonitor is not None:
            self.pointerMonitor.connect("activate", self.onPointerOutside)
        return False
        
    def onWindowMap( self, *args ):
        self.applet.set_state( Gtk.StateType.SELECTED )
        if self.keybinder == None or self.pointerMonitor == None:
            return
        
        self.keybinder.set_focus_window( self.mainwin.window.window )
        #self.pointerMonitor.grabPointer()
        return False
        
    def onWindowUnmap( self, *args ):
        self.applet.set_state( Gtk.StateType.NORMAL )
        if self.keybinder == None or self.pointerMonitor == None:
            return
        
        self.keybinder.set_focus_window()
        #self.pointerMonitor.ungrabPointer()
        return False
        
    def onRealize( self, *args):
        self.pointerMonitor.addWindowToMonitor( self.mainwin.window.window )
        self.pointerMonitor.addWindowToMonitor( self.applet.window )
        self.pointerMonitor.start()
        return False
     
    def onPointerOutside(self, *args):
        self.mainwin.hide()
        return True

    def onBindingPress(self, binder):
        self.toggleMenu()
        return True

    def changeIcon(self, settings, key, args = None):
        self.panel_size = self.settings_panel.get_int("size")
        self.pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size("/usr/lib/ubuntu-mate/mate-menu/start.svg", self.panel_size, self.panel_size)
        self.button_icon.set_from_pixbuf(self.pixbuf)
        self.button_box.set_size_request(self.panel_size + 20, -1)
        Gdk.flush()

    def createPanelButton( self ):
        self.settings_panel = Gio.Settings.new_with_path("org.mate.panel.toplevel", "/org/mate/panel/toplevels/bottom/")
        self.settings_panel.connect("changed::size", self.changeIcon)
        self.panel_size = self.settings_panel.get_int("size")

        self.pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size("/usr/lib/ubuntu-mate/mate-menu/start.svg", self.panel_size, self.panel_size)
         
        self.button_icon = Gtk.Image.new()
        self.button_icon.set_from_pixbuf(self.pixbuf)
        self.button_icon.set_tooltip_text(_("Start"))

        self.button_box = Gtk.HBox()
        self.button_box.set_size_request(self.panel_size + 20, -1)
        self.button_box.pack_start( self.button_icon, True, True, 0 )
        self.button_icon.set_padding( 0, 0 )

        self.button_box.set_homogeneous( False )
        self.button_box.show_all()

        self.applet.add( self.button_box )
        self.applet.set_background_widget( self.applet )

    def bind_hot_key (self):
        try:
            if self.hotkeyText != "":
                self.keybinder.grab( self.hotkeyText )
            self.keybinder.connect("activate", self.onBindingPress)
            self.keybinder.start()
            # Binding menu to hotkey
            print "Binding to Hot Key: " + self.hotkeyText
            
        except Exception, cause:
            print "** WARNING ** - Menu Hotkey Binding Error"
            print "Error Report :\n", str(cause)
            pass
    def showAboutDialog( self, action, userdata = None ):
        about = Gtk.AboutDialog()
        about.set_name("Ukui Menu")
        about.set_version(__VERSION__)
        about.set_comments( _("Startup Menu") )
        about.set_logo( GdkPixbuf.Pixbuf.new_from_file("/usr/lib/ubuntu-mate/mate-menu/start.svg") )
        about.connect( "response", lambda dialog, r: dialog.destroy() )
        about.show()

    def showPropertyDialog( self, action, userdata = None ):
        builder = Gtk.Builder()
        builder.add_from_file(os.path.join( '/', 'usr', 'lib', 'ubuntu-mate', 'mate-menu', "menu-property.glade" ))
        window = builder.get_object("window")
        window.set_default_size(230, 400)
        window.set_resizable(False)
        window.set_default_icon(GdkPixbuf.Pixbuf.new_from_file("/usr/lib/ubuntu-mate/mate-menu/start.svg"))

        window.set_title(_("Menu Property"))
        labelOpacity = builder.get_object("label_opacity")
        labelOpacity.set_text(_("Transparency:"))
        labelMenu = builder.get_object("label1")
        labelMenu.set_text(_("Menu Type:"))
        labelMenu.set_tooltip_text(_("This is used to set up the menu type."))
        labelRecent = builder.get_object("label2")
        labelRecent.set_text(_("File History:"))
        labelRecent.set_tooltip_text(_("This is used to set up whether the file history is displayed."))
        self.radioButton1 = builder.get_object("radiobutton1")
        self.radioButton1.set_label(_("Normal Menu"))
        self.radioButton2 = builder.get_object("radiobutton2")
        self.radioButton2.set_label(_("Category Menu"))
        if self.mainwin.showCategoryMenu:
            self.radioButton2.set_active(True)
        else:
            self.radioButton1.set_active(True)
        self.radioButton1.connect("button-press-event", self.radioToggled)
        self.radioButton2.connect("button-press-event", self.radioToggled)
        self.togglebutton = builder.get_object("togglebutton")
        state = self.settings.get_boolean("show-recent-file")
        if state:
            self.togglebutton.set_active(True)
            self.togglebutton.set_label(_("Opened"))
        else:
            self.togglebutton.set_active(False)
            self.togglebutton.set_label(_("Closed"))
        self.togglebutton.connect("toggled", self.Toggled)

        label3 = builder.get_object("label3")
        label3.set_text(_("Recent file number:"))
        label4 = builder.get_object("label4")
        label4.set_text(_("Recent app number:"))

        self.recent_file_num_bt = builder.get_object("recent_file_num_bt")
        if not state:
            self.recent_file_num_bt.set_sensitive(False)
        self.recent_file_num_bt.connect("value-changed", self.value_file_changed)
        recent_file_num = self.settings.get_int("recent-file-num")
        self.recent_file_num_bt.set_value(recent_file_num)

        self.recent_app_num_bt = builder.get_object("recent_app_num_bt")
        self.recent_app_num_bt.connect("value-changed", self.value_app_changed)
        recent_app_num = self.settings.get_int("recent-app-num")
        self.recent_app_num_bt.set_value(recent_app_num)

        self.opacity = builder.get_object("opacity")
        self.opacity.connect("value-changed", self.opacity_changed)
        opacity = self.mainwin.settings.get_double("opacity")
        self.opacity.set_value(opacity)

        self.resetButton = builder.get_object("button2")
        self.resetButton.set_label(_("Reset to default(_R)"))
        self.resetButton.set_use_underline(True)
        self.resetButton.connect("clicked", self.reset_to_default)

        self.closeButton = builder.get_object("button1")
        image = Gtk.Image.new_from_icon_name("gtk-close", Gtk.IconSize.MENU)
        self.closeButton.set_image(image)
        self.closeButton.set_label(_("Close(_C)"))
        self.closeButton.set_use_underline(True)
        self.closeButton.connect("clicked", lambda w: window.hide())
        window.show()

    def reset_to_default( self, widget ):
        self.settings.reset("show-category-menu")
        self.settings.reset("show-recent-file")
        self.settings.reset("recent-file-num")
        self.settings.reset("recent-app-num")
        self.settings.reset("opacity")
        state = self.settings.get_boolean("show-category-menu")
        if state:
            self.radioButton2.set_active(True)
        else:
            self.radioButton1.set_active(True)
        state = self.settings.get_boolean("show-recent-file")
        if state:
            self.togglebutton.set_active(True)
            self.togglebutton.set_label("已打开")
        else:
            self.togglebutton.set_active(False)
            self.togglebutton.set_label("已关闭")
        self.recent_file_num_bt.set_value(self.settings.get_int("recent-file-num"))
        self.recent_app_num_bt.set_value(self.settings.get_int("recent-app-num"))
        self.opacity.set_value(self.settings.get_double("opacity"))

    def value_file_changed( self, widget ):
        num = widget.get_value_as_int()
        self.settings.set_int("recent-file-num", num)

    def opacity_changed( self, widget ):
        value = widget.get_value()
        self.mainwin.settings.set_double("opacity", value)

    def value_app_changed( self, widget ):
        num = widget.get_value_as_int()
        self.settings.set_int("recent-app-num", num)

    def Toggled( self, user_data ):
        state = self.togglebutton.get_active()
        if state:
            self.settings.set_boolean("show-recent-file", True)
            self.recent_file_num_bt.set_sensitive(True)
            self.togglebutton.set_label("已打开")
        else:
            self.settings.set_boolean("show-recent-file", False)
            self.recent_file_num_bt.set_sensitive(False)
            self.togglebutton.set_label("已关闭")

    def radioToggled( self, widget, event ):
        if widget == self.radioButton1:
            self.settings.set_boolean("show-category-menu", False)
        elif widget == self.radioButton2:
            self.settings.set_boolean("show-category-menu", True)
        

    def reloadSettings( self, *args ):
        self.loadSettings()

    def showMenu( self, widget=None, event=None ):
        if self.mainwin is None:
            return 
        if event == None or event.button == 1:
            self.toggleMenu()
        # 开始菜单右键功能禁用
        elif event.button == 2:
            self.mainwin.hide( True )

    def toggleMenu( self ):
        if self.applet.state & Gtk.StateType.SELECTED:
            self.mainwin.hide( True )
        else:
            self.mainwin.show()

    def positionMenu( self ):
        # Get our own dimensions & position
        ourWidth  = self.mainwin.window.get_size()[0]
        ourHeight = self.mainwin.window.get_size()[1] + self.mainwin.offset

        x = c_int()
        y = c_int()
        # Get the dimensions/position of the widgetToAlignWith
	gdk.gdk_window_get_origin.restype = c_int
	gdk.gdk_window_get_origin.argtypes = [c_void_p, c_void_p, c_void_p]
        gdk.gdk_window_get_origin(hash(self.applet.window), byref(x), byref(y))
        entryX = x.value
        entryY = y.value

        entryWidth, entryHeight =  self.applet.get_allocation().width, self.applet.get_allocation().height
        entryHeight = entryHeight + self.mainwin.offset

        # Get the screen dimensions
        screenHeight = Gdk.Screen.height()
        screenWidth = Gdk.Screen.width()
        if self.applet.get_orient() == MatePanelApplet.AppletOrient.UP or self.applet.get_orient() == MatePanelApplet.AppletOrient.DOWN:
            if entryX + ourWidth < screenWidth or  entryX + entryWidth / 2 < screenWidth / 2:
            # Align to the left of the entry
                newX = entryX
            else:
                # Align to the right of the entry
                newX = entryX + entryWidth - ourWidth

            if entryY + entryHeight / 2 < screenHeight / 2:
                # Align to the bottom of the entry
                newY = entryY + entryHeight
            else:
                newY = entryY - ourHeight
        else:
            if entryX + entryWidth / 2 < screenWidth / 2:
                # Align to the left of the entry
                newX = entryX + entryWidth
            else:
                # Align to the right of the entry
                newX = entryX - ourWidth

            if entryY + ourHeight < screenHeight or entryY + entryHeight / 2 < screenHeight / 2:
                # Align to the bottom of the entry
                newY = entryY
            else:
                newY = entryY - ourHeight + entryHeight
        # -"Move window"
        self.mainwin.window.move( newX, newY )

    def changeMenuState(self, settings, key, args = None):
        if self.mainwin is None:
            return

        self.mainwin.RegenPlugins()

    def changeOpacity(self, settings, key, args = None):
        opacity = self.settings.get_double("opacity")
        self.mainwin.window.set_opacity(opacity)
 
    def LoadNormalMenu( self, *args, **kargs ):
        if self.mainwin is None:
            return

        self.settings.set_boolean("show-category-menu", False)
        os.system("killall mozo")
        self.state = False

    def LoadCategoryMenu( self, *args, **kargs ):
        if self.mainwin is None:
            return

        self.settings.set_boolean("show-category-menu", True)
        self.state = True

    def EditCategoryMenu( self, *args, **kargs ):
        os.system("mozo &")

    # this callback is to create a context menu
    def create_menu(self, widget, ev):
        if self.mainwin is None:
            return

        mTree = Gtk.Menu()
        mTree.set_events(Gdk.EventMask.POINTER_MOTION_MASK | Gdk.EventMask.POINTER_MOTION_HINT_MASK |
                         Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK)

        normalMenuItem = Gtk.ImageMenuItem.new_with_mnemonic(_("Normal Menu(_N)"))
        image = Gtk.Image.new_from_icon_name("gtk-new", Gtk.IconSize.MENU)
        normalMenuItem.set_image(image)

        categoryMenuItem = Gtk.ImageMenuItem.new_with_mnemonic(_("Category Menu(_C)"))
        image = Gtk.Image.new_from_icon_name("gtk-new", Gtk.IconSize.MENU)
        categoryMenuItem.set_image(image)

        separator1 = Gtk.SeparatorMenuItem()

        editMenuItem = Gtk.ImageMenuItem.new_with_mnemonic(_("Edit Category Menu(_E)"))
        image = Gtk.Image.new_from_icon_name("gtk-edit", Gtk.IconSize.MENU)
        editMenuItem.set_image(image)

        propertyMenuItem = Gtk.ImageMenuItem.new_with_mnemonic(_("Property(_P)"))
        image = Gtk.Image.new_from_icon_name("gtk-properties", Gtk.IconSize.MENU)
        propertyMenuItem.set_image(image)


        mTree.append(normalMenuItem)
        mTree.append(categoryMenuItem)
        mTree.append(separator1)
        #mTree.append(editMenuItem)
        mTree.append(propertyMenuItem)

        mTree.show_all()
        state = self.settings.get_boolean("show-category-menu")
        if state:
            normalMenuItem.set_visible(True)
            categoryMenuItem.set_visible(False)
            editMenuItem.set_visible(True)
        else:
            normalMenuItem.set_visible(False)
            categoryMenuItem.set_visible(True)
            editMenuItem.set_visible(False)

        normalMenuItem.connect("activate", self.LoadNormalMenu, widget)
        categoryMenuItem.connect("activate", self.LoadCategoryMenu, widget)
        editMenuItem.connect("activate", self.EditCategoryMenu, widget)
        propertyMenuItem.connect("activate", self.showPropertyDialog, widget)

        gtk.gtk_menu_popup.argtypes = [ctypes.c_ulong, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, ctypes.c_long]
        gtk.gtk_menu_popup(hash(mTree), None, None, None, None, ev.button, ev.time)

def applet_factory( applet, iid, data ):
    MenuWin( applet, iid )
    applet.show()
    return True

def quit_all(widget):
    Gtk.main_quit()
    sys.exit(0)

MatePanelApplet.Applet.factory_main("MateMenuAppletFactory", True,
                                    MatePanelApplet.Applet.__gtype__,
                                    applet_factory, None)
